from django.db import models
from django.utils.translation import gettext_lazy as _
from solo.models import SingletonModel


class DiscordIntegration(SingletonModel):
    bot_name = models.CharField(max_length=30, blank=True,
                                verbose_name=_('discord bot username'),
                                help_text=_('Override the bot username set in '
                                            'the Discord settings.'))
    avatar_url = models.URLField(blank=True,
                                 verbose_name=_('discord bot avatar'),
                                 help_text=_('Override the bot avatar set in '
                                             'the Discord settings.'))
    webhook_url = models.URLField(verbose_name=_('discord webhook url'),
                                  help_text=_('The Discord webhook url found '
                                              'in the Discord settings.'))



import datetime
import json

import requests

from django.core.exceptions import ImproperlyConfigured

from discord_integration.models import DiscordIntegration

__all__ = ['discord_message']


def timestamp():
    return datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc) \
                            .replace(microsecond=0).isoformat()


def discord_message(message):
    data = DiscordIntegration.get_solo()

    if not data.webhook_url:
        return ImproperlyConfigured('Discord Webhook URL is not configured')

    if data.bot_name and 'username' not in message:
        message['username'] = data.bot_name

    if data.avatar_url and 'avatar_url' not in message:
        message['avatar_url'] = data.avatar_url

    if 'timestamp' not in message:
        message['timestamp'] = timestamp()

    requests.post(data.webhook_url, data=json.dumps(message),
                  headers={'Content-Type': 'application/json'})



from django.contrib import admin
from solo.admin import SingletonModelAdmin

from discord_integration.models import DiscordIntegration


class DiscordIntegrationAdmin(SingletonModelAdmin):
    exclude = ()


admin.site.register(DiscordIntegration, DiscordIntegrationAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-12 19:03
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DiscordIntegration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bot_name', models.CharField(blank=True, help_text='Override the bot username set in the Discord settings.', max_length=30, verbose_name='discord bot username')),
                ('avatar_url', models.URLField(blank=True, help_text='Override the bot avatar set in the Discord settings', verbose_name='discord bot avatar')),
                ('webhook_url', models.URLField(help_text='The Discord webhook url found in the Discord settings', verbose_name='discord webhook url')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.conf import settings
from django.utils.log import AdminEmailHandler

__all__ = ['DiscordMessageHandler', 'SimpleDiscordMessageHandler']


MESSAGE_LIMIT = getattr(settings, 'DISCORD_MESSAGE_LIMIT', 1000)


def escape(text):
    escape_chars = ('*', '_', '~', '`', '|')
    for ch in escape_chars:
        text = text.replace(ch, '\\' + ch)
    return text


class DiscordMessageHandler(AdminEmailHandler):
    def emit(self, record):
        from discord_integration.models import DiscordIntegration

        data = DiscordIntegration.get_solo()

        if not data.webhook_url:
            return

        self.__level = record.levelname
        super().emit(record)

    def send_mail(self, subject, message, *args, **kwargs):
        from discord_integration.message import discord_message

        colors = {
            'ERROR': 0xe74c3c,
            'WARNING': 0xe78c3c,
            'INFO': 0xe7ec3c,
        }

        discord_message({
            'embeds': [{
                'title': escape(subject),
                'description': escape(message[:MESSAGE_LIMIT]),
                'color': colors.get(self.__level, 0xeee),
            }],
        })


class SimpleDiscordMessageHandler(DiscordMessageHandler):
    def send_mail(self, subject, message, *args, **kwargs):
        super().send_mail(subject, '', *args, **kwargs)
